#include "xmlloadmodel.h"
#include "stdafx.h"

#include <iostream>
#include <sstream>

// #define DEBUG_XMLLOAD

void XmlSpitReadingInfo(CvFileStorage * fs, CvFileNode * node, char const * type, char const * name)
{
#ifdef DEBUG_XMLLOAD
    std::cout << "Reading " << type << " by name(" << name << ") from " << (void*)node << std::flush;
    if (node)
        std::cout << "(" << cvGetFileNodeName(node) << ")" << std::flush;
    std::cout << "\n" << std::flush;
#endif
}

double myReadRealByName(CvFileStorage * fs, CvFileNode * node, char const * name)
{
    XmlSpitReadingInfo(fs, node, "real", name);
#ifdef DEBUG_XMLLOAD
    std::cout << "real(" << cvReadRealByName(fs, node, name) << ")\n" << std::flush;
#endif
    return cvReadRealByName(fs, node, name);
}

int myReadIntByName(CvFileStorage * fs, CvFileNode * node, char const * name)
{
    XmlSpitReadingInfo(fs, node, "int", name);
#ifdef DEBUG_XMLLOAD
    std::cout << "int(" << cvReadIntByName(fs, node, name) << ")\n" << std::flush;
#endif
    return cvReadIntByName(fs, node, name);
}

char const * myReadStringByName(CvFileStorage * fs, CvFileNode * node, char const * name)
{
    XmlSpitReadingInfo(fs, node, "string", name);
#ifdef DEBUG_XMLLOAD
    std::cout << "string(" << cvReadStringByName(fs, node, name) << ")\n" << std::flush;
#endif
    return cvReadStringByName(fs, node, name);
}

CvFileNode * myGetFileNodeByName(CvFileStorage * fs, CvFileNode * node, char const * name)
{
    XmlSpitReadingInfo(fs, node, "node", name);
    return cvGetFileNodeByName(fs, node, name);
}

void SpitNodeInfo( CvFileNode * node )
{
#ifdef DEBUG_XMLLOAD
    std::cout << "node(" << (void*)node << ") " << std::flush;
    if (node)
        std::cout << cvGetFileNodeName(node);
    std::cout << "\n" << std::flush;
#endif
}

IMPEXP
void xmlloadmodel(std::string const & file, Model & model)
{
    CvFileStorage* fs = cvOpenFileStorage(file.c_str(), NULL, CV_STORAGE_READ);
	if (!fs) {
		printf("Could not read classifier %s\n",file.c_str());
		exit(0);
	}
    CvFileNode * rootModelNode = myGetFileNodeByName(fs, NULL, "model");
    SpitNodeInfo( rootModelNode );
	// assert( std::string("model") == cvGetFileNodeName(rootModelNode) );
    assert( std::string("adacascadeclass") == myReadStringByName(fs, rootModelNode, "fun") );
    SpitNodeInfo( rootModelNode );
    int stage_count = myReadIntByName(fs, rootModelNode, "stagemodel_count");
	int histblock_winsize_x, histblock_size_x;
	int histblock_winsize_y, histblock_size_y;
	if (cvGetFileNodeByName(fs, rootModelNode, "histblock_winsize_x") && 
		cvGetFileNodeByName(fs, rootModelNode, "histblock_winsize_y") &&
		cvGetFileNodeByName(fs, rootModelNode, "histblock_size_x") &&
		cvGetFileNodeByName(fs, rootModelNode, "histblock_size_y")) {
		histblock_winsize_x = myReadIntByName(fs, rootModelNode, "histblock_winsize_x");
		histblock_winsize_y = myReadIntByName(fs, rootModelNode, "histblock_winsize_y");
		histblock_size_x = myReadIntByName(fs, rootModelNode, "histblock_size_x");
		histblock_size_y = myReadIntByName(fs, rootModelNode, "histblock_size_y");
	} else {
		histblock_winsize_x = 20; // default x-size value
		histblock_winsize_y = 10; // default y-size value
		histblock_size_x = 5; // default x-block size
		histblock_size_y = 5; // default y-block size
	}

	char histtype[256];
	if (cvGetFileNodeByName(fs, rootModelNode, "histtype"))
		strcpy(histtype,myReadStringByName(fs, rootModelNode, "histtype"));
	else
		strcpy(histtype,"orienthistmod"); // default histogram type

	char label[256];
	if (cvGetFileNodeByName(fs, rootModelNode, "label"))
		strcpy(label,myReadStringByName(fs, rootModelNode, "label"));
	else
		strcpy(label,"unknown"); // default label

	//printf("parameters: %d,%d %d,%d hittype: '%s'\n",histblock_winsize_x,histblock_winsize_y,
	//	histblock_size_x,histblock_size_y,histtype);

	assert( stage_count >= 0 );
    for (int stagei = 0; stagei < stage_count; ++stagei)
    {
        std::stringstream buf;
        buf << "stagemodel" << (stagei + 1);
        CvFileNode * stageNode = myGetFileNodeByName( fs, rootModelNode, buf.str().c_str() );
        SpitNodeInfo( stageNode );
        assert( stageNode );
        Stage stage;
        stage.stage_threshold = myReadRealByName(fs, stageNode, "stage_threshold");
        stage.exitflag = myReadIntByName(fs, stageNode, "exitflag");
		stage.histblock_winsize_x=histblock_winsize_x;
		stage.histblock_winsize_y=histblock_winsize_y;
		stage.histblock_size_x=histblock_size_x;
		stage.histblock_size_y=histblock_size_y;
		strcpy(stage.histtype,histtype);
		strcpy(stage.label,label);

        int rule_count = myReadIntByName(fs, stageNode, "member_count");
        assert( rule_count >= 0 );
        for (int rulei = 0; rulei < rule_count; ++rulei)
        {
            std::stringstream rulebuf;
            rulebuf << "member" << (rulei + 1);
            CvFileNode * memberNode = cvGetFileNodeByName( fs, stageNode, rulebuf.str().c_str() );
            assert( memberNode );
            StageRule rule;
            rule.Alpha = myReadRealByName(fs, memberNode, "Alpha");
            rule.Z = myReadRealByName(fs, memberNode, "Z");
            rule.WeightedErr = myReadRealByName(fs, memberNode, "WeightedErr");
            rule.ErrBound = myReadRealByName(fs, memberNode, "ErrBound");

            CvFileNode * ruleNode = cvGetFileNodeByName( fs, memberNode, "rule" );
            assert( ruleNode );
            rule.type = (StageRuleType)myReadIntByName(fs, ruleNode, "ftype");
            rule.trainerror = myReadRealByName(fs, ruleNode, "trainerror");

            CvFileNode * rectNode = cvGetFileNodeByName( fs, ruleNode, "rect" );
            assert( rectNode );
            // Note: it seems we dont need that // '-1' is due to the fact that matlab has 1-based indexes instead of 0-based
            rule.left   = myReadIntByName(fs, rectNode, "x"); // - 1;
            rule.top    = myReadIntByName(fs, rectNode, "y"); // - 1;
            rule.width  = myReadIntByName(fs, rectNode, "w");
            rule.height = myReadIntByName(fs, rectNode, "h");

            CvFileNode * ruleModelNode = cvGetFileNodeByName( fs, ruleNode, "rule_model" );
            assert( ruleModelNode );
            rule.separab = myReadRealByName(fs, ruleModelNode, "separab");
            rule.b = myReadRealByName(fs, ruleModelNode, "b");
            
            CvFileNode * wNode = cvGetFileNodeByName( fs, ruleModelNode, "W" );
            assert( wNode );
            uint wCount = myReadIntByName(fs, wNode, "w_count");
            for (uint wi = 0; wi < wCount; ++wi)
            {
                std::stringstream wbuf;
                wbuf << "w" << (wi + 1);
                rule.W.push_back( myReadRealByName(fs, wNode, wbuf.str().c_str()) );
            }
            int size = 0;
            if (rule.type == SrtOneSquare)
                size = 4; //12;
            else if (rule.type == SrtTwoCols || rule.type == SrtTwoRows)
                size = 8; //24;
            else if (rule.type == SrtFourQuarters)
                size = 16; //48;
            else
            {
                assert( rule.type == (int)"unknown" );
            }
			int rws=rule.W.size();
            assert( size == rule.W.size() );
            stage.rules.push_back( rule );
        }
        model.push_back( stage );
    }
}
